import json
import os
import boto3
import threading
from time import sleep
from ..encryption import Encryption


class Sqs:

    def __init__(self, QueueName=None, Attributes={"VisibilityTimeout": "60"}):
        self.ENVIRONMENT = json.loads(os.getenv("CLUSTER_CONFIG")).get("environment")
        self.kong_api = json.loads(os.getenv("CLUSTER_CONFIG")).get("kong-admin-url")

        boto3.setup_default_session(region_name='eu-central-1')
        self.sqs = boto3.resource('sqs')
        self.sqs_client = boto3.client('sqs', region_name='eu-central-1')
        self.cipher = json.loads(os.getenv("CLUSTER_CONFIG")).get("cipher-key")
        self.crypt = Encryption()

        if QueueName is not None:
            self.set_queue(QueueName=QueueName, Attributes=Attributes)

    def get_all_queues(self):
        for queue in self.sqs.queues.all():
            print(queue.url)

    def set_queue(self, QueueName, Attributes=None):
        try:
            self.queue = self.sqs.get_queue_by_name(QueueName=QueueName)
        except Exception as e:
            if hasattr(e, "response") and e.response.get("Error").get("Code") == "AWS.SimpleQueueService.NonExistentQueue":
                print("Trying to auto-create queue...")
                self.create_queue(QueueName=QueueName, Attributes=Attributes)
            else:
                raise
        return True

    def create_queue(self, QueueName, Attributes={}):
        try:
            self.queue = self.sqs.create_queue(QueueName=QueueName, Attributes=Attributes)
            self.queue_url = self.sqs_client.get_queue_url(QueueName=QueueName)['QueueUrl']
            self.queue_attrs = self.sqs_client.get_queue_attributes(QueueUrl=self.queue_url, AttributeNames=['All'])[
                'Attributes']
            self.queue_arn = self.queue_attrs['QueueArn']
            if ':sqs.' in self.queue_arn:
                self.queue_arn = self.queue_arn.replace(':sqs.', ':')

        except Exception as e:
            if hasattr(e, 'response') and e.response.get("Error").get("Code") == "AWS.SimpleQueueService.QueueDeletedRecently":
                print(e.response)
                print("Waiting 60 seconds and try again automatically...")
                sleep(62)
                self.queue = self.sqs.create_queue(QueueName=QueueName, Attributes=Attributes)
                self.queue_url = self.sqs_client.get_queue_url(QueueName=QueueName)['QueueUrl']
                self.queue_attrs = \
                self.sqs_client.get_queue_attributes(QueueUrl=self.queue_url, AttributeNames=['All'])[
                    'Attributes']
                self.queue_arn = self.queue_attrs['QueueArn']
                if ':sqs.' in self.queue_arn:
                    self.queue_arn = self.queue_arn.replace(':sqs.', ':')
            else:
                raise

        print("Queue '{0}' created successfully!".format(QueueName))

    def subscribe_to_topic(self, topic):
        # Subscribe SQS queue to SNS
        response = self.sqs_client.set_queue_attributes(
            QueueUrl=self.queue_url,
            Attributes={
                "Policy": self.allow_sns_to_write_to_sqs()
            }
        )

        sns_client = boto3.client('sns', region_name='eu-central-1')
        response = sns_client.subscribe(
            TopicArn="arn:aws:sns:eu-central-1:282615081127:" + topic,
            Protocol='sqs',
            Endpoint=self.queue_arn
        )

        print(response)

    def create_topic(self,topic):
        sns_client = boto3.client('sns', region_name='eu-central-1')
        topic_res = sns_client.create_topic(Name=topic)
        sns_topic_arn = topic_res['TopicArn']
        return sns_topic_arn

    def send(self, MessageBody, MessageAttributes={}):
        if type(MessageBody) == dict:
            MessageBody = json.dumps(MessageBody)
            MessageBody = self.crypt.encrypt(MessageBody, self.cipher)
        self.queue.send_message(MessageBody=MessageBody, MessageAttributes=MessageAttributes)

        return True

    def receive(self, callback=None, MaxNumberOfMessages=10, endless=False):
        while (1):
            for message in self.queue.receive_messages(MaxNumberOfMessages=MaxNumberOfMessages):
                if callback is not None and callable(callback):
                    result = callback(message)
                elif isinstance(callback, object) and type(self) in callback.__class__.__bases__:
                    self.load_message(message)
                    result = self.handleTopic(callback)
                if result == True:
                    message.delete()
            if endless is not True:
                break
            else:
                sleep(3)

    def handleMessage(self, message):
        print(
            "No Message-Handler-Provided. Please use this class, extend from it and implement your own handleMessage funktion")
        return False

    def load_message(self, message):
        body = self.crypt.decrypt(message.body, self.cipher)
        self.msg = json.loads(body)

    def handleTopic(self, callback):
        topic = self.getTopic()
        if hasattr(callback, topic):
            method_to_call = getattr(callback, self.getTopic())
            return method_to_call(self.msg.get("Message"))
        else:
            print("No handler for topic: " + topic)
            return False

    def getTopic(self):
        topic_arn = self.msg.get("TopicArn")
        return topic_arn.split(':')[-1]

    def addConsumer(self, callback=None, MaxNumberOfMessages=10):
        t = threading.Thread(target=self.receive, args=([callback, MaxNumberOfMessages, True]))
        t.daemon = True
        t.start()

    def allow_sns_to_write_to_sqs(self):
        policy_document = {
            "Version": "2012-10-17",
            "Id": self.queue_arn + "/SQSDefaultPolicy",
            "Statement": [
                {
                    "Sid": "Sid1548944236219",
                    "Effect": "Allow",
                    "Principal": "*",
                    "Action": "SQS:SendMessage",
                    "Resource": self.queue_arn,
                    "Condition": {
                        "ArnEquals": {
                            "aws:SourceArn": "arn:aws:sns:eu-central-1:282615081127:*"
                        }
                    }
                }
            ]
        }

        return json.dumps(policy_document)



import boto3
import json

class Sns:
    def publish_event(self, correlationId, eventKind, payload):
        payload["correlationId"] = correlationId
        sns_client = boto3.client('sns', region_name='eu-central-1')

        sns_topic_arn = "arn:aws:sns:eu-central-1:282615081127:" + str(eventKind)
        response = sns_client.publish(
            TopicArn=sns_topic_arn,
            Message=json.dumps(payload),
            MessageStructure='raw')

        return response



import socket
from datetime import datetime
import json

class Logger:
    def __init__(self, host, port, consoleLog=False):
        self.host = host
        self.port = port
        self.consoleLog = consoleLog

    def getLogger(self):
        return self.log

    def log(self, message, kind="INFO"):
        now = datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S.%f')
        msg = {"source": socket.gethostname(), "time": now, "kind": kind, "message": message}
        if self.consoleLog == True:
            print(json.dumps(msg))
        else:
            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            sock.connect((self.host, self.port))
            try:
                sock.sendall(bytes(json.dumps(msg), 'utf-8'))
            finally:
                sock.close()



import os
import cherrypy
from inspect import getframeinfo, stack


class Server:

    def start(self, flask_app=None, app_path="/", static_dir="/static", static_path=None, port=8000):
        # Mount the application
        cherrypy.tree.graft(flask_app, app_path)

        class Static(object):
            pass

        if static_dir is not None and static_path is None:
            static_path = static_dir
        if static_dir is not None and static_path is not None:
            # Add static file serving to enable swagger GUI
            caller = getframeinfo(stack()[1][0])
            PATH = os.path.abspath(os.path.dirname(caller.filename))
            cherrypy.tree.mount(Static(), static_path, config={
                '/': {
                    'tools.staticdir.on': True,
                    'tools.staticdir.dir': PATH + static_dir,
                    'tools.staticdir.index': 'index.html',
                }
            })

        # Unsubscribe the default server
        cherrypy.server.unsubscribe()

        # Instantiate a new server object
        server = cherrypy._cpserver.Server()

        # Configure the server object
        server.socket_host = "0.0.0.0"
        server.socket_port = port
        server.thread_pool = 30

        # Subscribe this server
        server.subscribe()

        # Start the server engine (Option 1 *and* 2)
        cherrypy.engine.start()
        cherrypy.engine.block()


if __name__ == '__main__':
    server = Server()
    server.start()



import requests
import json
import os


class Autorun:

    def __init__(self):
        self.ENVIRONMENT = json.loads(os.getenv("CLUSTER_CONFIG")).get("environment")
        self.kong_api = json.loads(os.getenv("CLUSTER_CONFIG")).get("kong-admin-url")
        print("Detected environment:", self.ENVIRONMENT)
        print("Using " + str(self.kong_api) + "as api-gateway")

    def post_deployment(self):
        ENVIRONMENT = json.loads(os.getenv("CLUSTER_CONFIG")).get("environment")
        KONG_ADMIN = json.loads(os.getenv("CLUSTER_CONFIG")).get("kong-admin-url")
        print("Detected environment:", ENVIRONMENT)

        request = requests.get(self.kong_api)
        if request.status_code == 200:
            self.add_service("Deployment-Service", "http://openshift-deployment-pipeline.planblick.svc:8000")
            self.add_route(service_name="Deployment-Service", path="/deployment")
        else:
            print("Kong-API not available:", str(self.kong_api))

    def add_service(self, service_name, service_url):
        payload = {
            "name": service_name,
            "url": service_url
        }
        headers = {
            'Content-Type': "application/json",
            'cache-control': "no-cache",
        }
        url = self.kong_api + "/services"
        response = requests.request("GET", url, headers=headers)

        services = response.json()
        for service in services.get("data", []):
            if service_name == service.get("name"):
                print("Service already exists")
                return

        self.post(api_path="/services", payload=payload, headers=headers)

    def add_route(self, service_name, path, plugins=["key-auth"], strip_path=True):
        payload = {
            "paths": [path],
            "strip_path": strip_path,
        }
        headers = {
            'Content-Type': "application/json",
            'cache-control': "no-cache",
        }

        url = self.kong_api + "/services/" + service_name + "/routes/"

        response = requests.request("GET", url, headers=headers)

        routes = response.json()
        route_id = None
        path_exists = False
        for route in routes.get("data", []):
            if path in route.get("paths"):
                print("Path already exists")
                route_id = route.get("id")
                path_exists = True

        if path_exists is False:
            reply = self.post(api_path="/services/" + service_name + "/routes", payload=payload, headers=headers)
            route_id = reply.get("id")

        url = self.kong_api + "/plugins"
        for plugin in plugins:
            payload = {"route_id": route_id, "name": plugin}
            headers = {
                'Content-Type': "application/json",
                'cache-control': "no-cache",
            }

            try:
                response = requests.request("POST", url, data=json.dumps(payload), headers=headers)
                print(response.text)
            except Exception:
                pass

    def post(self, api_path, payload, headers):
        if type(payload) == dict:
            payload = json.dumps(payload)

        url = self.kong_api + api_path
        response = requests.request("POST", url, data=payload, headers=headers)
        print(response.text)
        return response.json()



from setuptools import setup
version = "0.0.67"
setup(name='planblick',
      version=version,
      description='',
      url='https://www.planblick.com',
      author='Florian Kröber @ Planblick',
      author_email='fk@planblick.com',
      license='MIT',
      packages=['planblick.httpserver', 'planblick.autorun', 'planblick.sqs', 'planblick.sns', 'planblick.logger'],
      install_requires=[
          'cherrypy',
          'requests',
          'boto3',
          'cryptography'
      ],
      zip_safe=False)
